﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static ContainerShip.EntityShip;

namespace ContainerShip
{
    public class DrawingShip
    {
        public EntityShip Ship { protected set; get; }
        protected float _startPosX;
        protected float _startPosY;
        private int? _pictureWidth = null;
        private int _startPosXInt;
        private int _startPosYInt;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        /// <summary>
        /// Ширина отрисовки автомобиля
        /// </summary>
        private readonly int _shipWidth = 135;
        /// <summary>
        /// Высота отрисовки автомобиля
        /// </summary>
        private readonly int _shipHeight = 40;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес автомобиля</param>
        /// <param name="bodyColor">Цвет кузова</param>
        public DrawingShip(int speed, float weight, Color bodyColor)
        {
            Ship = new EntityShip(speed, weight, bodyColor);
        }

        protected DrawingShip(int speed, float weight, Color bodyColor, int
                shipWidth, int shipHeight) :
            this(speed, weight, bodyColor)
        {
            _shipWidth = shipWidth;
            _shipHeight = shipHeight;
        }
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void SetPosition(int x, int y, int width, int height)
        {
            // Проверки
            Random rn = new Random();
            if (x < 0 || x + _shipWidth> width)
            {
                x = rn.Next(0, width - _shipWidth);
            }
            if (y < 0 || y + _shipHeight > height)
            {
                y = rn.Next(0, height - _shipHeight);
            }
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
            
        }
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                case Direction.Right:
                    if (_startPosX + _shipWidth + Ship.Step < _pictureWidth)
                    {
                        _startPosX += Ship.Step;
                    }
                    break;
                case Direction.Left:
                    if (_startPosX - Ship.Step > 0)
                    {
                        _startPosX -= Ship.Step;
                    }
                    break;
                case Direction.Up:
                    if (_startPosY - Ship.Step > 0)
                    {
                        _startPosY -= Ship.Step;
                    }
                    break;
                case Direction.Down:
                    if (_startPosY + _shipHeight + Ship.Step < _pictureHeight)
                    {
                        _startPosY += Ship.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Отрисовка 
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
            || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            _startPosXInt = Convert.ToInt32(_startPosX);
            _startPosYInt = Convert.ToInt32(_startPosY);
            Pen pen = new(Color.Black);

            Point[] circuit =
                {
                new Point(_startPosXInt + 30, _startPosYInt),
                new Point(_startPosXInt + 100, _startPosYInt),
                new Point(_startPosXInt + 100, _startPosYInt + 15),
                new Point(_startPosXInt + 135, _startPosYInt + 15),
                new Point(_startPosXInt + 115 , _startPosYInt + 40),
                new Point(_startPosXInt + 20, _startPosYInt + 40),
                new Point(_startPosXInt, _startPosYInt + 15),
                new Point(_startPosXInt + 30, _startPosYInt + 15),
                new Point(_startPosXInt + 30, _startPosYInt),
                };
            Brush br = new SolidBrush(Ship?.BodyColor ?? Color.Black);
            g.FillPolygon(br, circuit);
            g.DrawPolygon(pen, circuit);
            g.DrawLine(pen, _startPosXInt + 30, _startPosYInt + 15, _startPosXInt + 100, _startPosYInt + 15);
            g.DrawLine(pen, _startPosXInt + 30, _startPosYInt + 20, _startPosXInt + 30, _startPosYInt + 30);
            g.DrawLine(pen, _startPosXInt + 25, _startPosYInt + 25, _startPosXInt + 35, _startPosYInt + 25);
            g.DrawLine(pen, _startPosXInt + 25, _startPosYInt + 30, _startPosXInt + 35, _startPosYInt + 30);
        }
        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _shipWidth || _pictureHeight <= _shipHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _shipWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _shipWidth;
            }
            if (_startPosY + _shipHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _shipHeight;
            }
        }

        /// <summary>
        /// Получение текущей позиции объекта
        /// </summary>
        /// <returns></returns>
        public (float Left, float Top, float Right, float Bottom) GetCurrentPosition()
        {
            return (_startPosX, _startPosY, _startPosX + _shipWidth, _startPosY + _shipHeight);
        }
    }
}
